#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <DHT.h>

DHT dht(PIN_PC3, DHT11);
LiquidCrystal_I2C lcd (0x27, 16, 2);

unsigned long time_now = 0;

void setup() {
  pinMode(PIN_PD3, OUTPUT);
  pinMode(PIN_PB0, OUTPUT);
  pinMode(PIN_PB1, OUTPUT);
  pinMode(PIN_PB2, OUTPUT);
  pinMode(PIN_PC2, INPUT);
  pinMode(PIN_PC3, INPUT);
  lcd.begin();
  dht.begin();
}

void set_pled(uint8_t x) {
  if (x & 0b001) {
    digitalWrite(PIN_PB0, HIGH);
  } else {
    digitalWrite(PIN_PB0, LOW);
  }

  if (x & 0b010) {
    digitalWrite(PIN_PB1, HIGH);
  } else {
    digitalWrite(PIN_PB1, LOW);
  }

  if (x & 0b100) {
    digitalWrite(PIN_PB2, HIGH);
  } else {
    digitalWrite(PIN_PB2, LOW);
  }
}

int set_lcd(int t, float data) {
  lcd.clear();
  lcd.home();

  if (t == 0) {
    lcd.print("Humidity: ");
    lcd.setCursor(0,1);
    lcd.print(data);
    lcd.print("%RH");
    return 0;
  } else if (t == 1) {
    lcd.print("Temperature: ") ;
    lcd.setCursor(0,1);
    lcd.print(data);
    lcd.print((char)0b11011111);
    lcd.print("C");
  } else if (t == 2) {
    lcd.print("Temperature: ") ;
    lcd.setCursor(0,1);
    lcd.print(data);
    lcd.print((char)0b11011111);
    lcd.print("F");
  } else if (t == 3) {
    lcd.print("UV Index: ") ;
    lcd.setCursor(0,1);
    lcd.print(data);
  } else if (t == 4) {
    lcd.print("Heat Index: ") ;
    lcd.setCursor(0,1);
    lcd.print(data);
    lcd.print((char)0b11011111);
    lcd.print("C");
  } else if (t == 5) {
    lcd.print("Heat Index: ") ;
    lcd.setCursor(0,1);
    lcd.print(data);
    lcd.print((char)0b11011111);
    lcd.print("F");
  } else if (t == -1) {
    lcd.print("ERROR!!!");
  }
}

void beacon(int x) {
    if (x) {
      for (int i=0;i<3;i++) {
        digitalWrite(PIN_PD3, HIGH);
        delay(100);
        digitalWrite(PIN_PD3, LOW);  
        delay(100);
      }
    } else {
      for (int i=0;i<3;i++) {
        digitalWrite(PIN_PD3, HIGH);
        delay(200);
        digitalWrite(PIN_PD3, LOW);  
        delay(200);
      }
    }
}

uint8_t cond(float hi, float uvi) {
    int a,b;
  
    if (hi<=80) {
      a = 1;
    } else if (hi<=103) {
      a = 2;
    } else {
      a = 4;
    }

    if (uvi<3) {
      b = 1;
    } else if (uvi<8) {
      b = 2;
    } else {
      b = 4;  
    }

    return max(a,b);
}

void disp(float h, float c, float f, float uv, float hic, float hif) {
  set_lcd(0,h);
  delay(1000);
  set_lcd(1,c);
  delay(1000);
  set_lcd(2,f);
  delay(1000);
  set_lcd(3,uv);
  delay(1000);
  set_lcd(4,hic);
  delay(1000);
  set_lcd(5,hif);
  delay(1000);
}

void loop() {
  beacon(1);
  float h = dht.readHumidity();
  float c = dht.readTemperature();
  float f = dht.readTemperature(true);

  if (isnan(h) || isnan(c) || isnan(f)) {
    set_lcd(-1,0);
  }

  float hif = dht.computeHeatIndex(f, h);
  float hic = dht.computeHeatIndex(c, h, false);

  float uv_read = analogRead(PIN_PC2);
  float uv_V = 5 * uv_read/1023;
  float uv = uv_V / 0.1;

  beacon(0);

  set_pled(cond(hif,uv));
  
  if (millis() >= time_now + 20000) {
    set_lcd(5,hif);
    time_now = millis();
  } else if (millis() >= time_now + 16500) {
    set_lcd(4,hic);
  } else if (millis() >= time_now + 13000) {
    set_lcd(3,uv);
  } else if (millis() >= time_now + 8500) {
    set_lcd(2,f);
  } else if (millis() >= time_now + 5000) {
    set_lcd(1,c);
  } else if (millis() >= time_now + 1500) {
      set_lcd(0,h);
  }
  delay(2000);
}
